# -*- coding: utf-8 -*-
"""
LimonAppDelegate.py - application delegate and main controller.

@Copyright (C) 2005 Nir Soffer <nirs@freeshell.org>
@license: GNU GPL, see COPYING for details.
"""

from Foundation import *
from AppKit import *
from PyObjCTools import NibClassBuilder
import objc

import dictionary


def serviceSelector(function):
    """ Decorate function with service selector signature 
    
    Needed to return the error correctly from service methods. In Cocoa, 
    a service get a pointer to an error string, and set the string to some
    value in case of error. Service method in Python does not get this 
    pointer and return a string with the error, or None.
    
    Copied from /Developer/Python/PyObjC/Examples/AppKit/SimpleService.
    """
    return objc.selector(function, signature="v@:@@o^@")


class LimonAppDelegate(NibClassBuilder.AutoBaseClass):
    """ Limon application delegate 
    
    I'm handling the application and the search interface, and use a 
    DictionaryView to define words and display results.
    
    I also provide "Define in Limon" service.    
    """
    # ---------------------------------------------------------------------
    # Application delegate
        
    def awakeFromNib(self):
        self.dictionaryView.setString_(u'')

    def validateMenuItem_(self, item):
        action = item.action()
        if action == 'new:':
            return self.searchField.stringValue().strip() != u''
        elif action == 'edit:':
            return (self.dictionaryView.word == self.searchField.stringValue()
                    and self.dictionaryView.hasDefinition)
        elif action == 'defineSelection:':
            return self.dictionaryView.selection() != u''            
        return False

    # ---------------------------------------------------------------------
    # Window delegate
    
    def windowWillClose_(self, notification):
        NSApp().terminate_(self)

    # ---------------------------------------------------------------------
    # Actions

    def new_(self, sender):
        dictionary.new(self.searchField.stringValue().UTF8String())

    def edit_(self, sender):
        dictionary.edit(self.searchField.stringValue().UTF8String())

    def defineSelection_(self, sender):
        self.searchField.setStringValue_(self.dictionaryView.selection())
        self.define_(sender)

    def define_(self, sender):
        self.searchField.selectText_(self)
        entry = self.searchField.stringValue()
        if entry:
            self.progressIndicator.startAnimation_(self)
            self.dictionaryView.define(entry)
            self.progressIndicator.stopAnimation_(self)
            
    # ---------------------------------------------------------------------
    # Handling the search field
    
    def controlTextDidEndEditing_(self, notification):
        self.searchField.setStringValue_(self.searchField.stringValue().strip())

    # ---------------------------------------------------------------------
    # Providing services

    def applicationDidFinishLaunching_(self, notification):
        NSApp().setServicesProvider_(self)

    def defineService_userData_error_(self, pboard, data):
        types = pboard.types()
        pboardString = None
        if NSStringPboardType in types:
            pboardString = pboard.stringForType_(NSStringPboardType)
        if pboardString is None:
            return NSLocalizedString(
                "Error: Pasteboard doesn't contain a string.", None)
        
        self.searchField.setStringValue_(pboardString)
        self.define_(self)
    
    defineService_userData_error_ = serviceSelector(
        defineService_userData_error_)
    